from __future__ import absolute_import, division, print_function, unicode_literals
import numpy as np
import pandas as pd
import os
import matplotlib
matplotlib.use("TkAgg")
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2TkAgg
from matplotlib import style
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import matplotlib.ticker as mtick

try:
    import Tkinter as tk
    import ttk
    import tkFont
    import tkMessageBox
except:
    import tkinter as tk
    from tkinter import ttk
    from tkinter import font as tkFont
    from tkinter import messagebox as tkMessageBox

from auquanToolbox.metrics import metrics, baseline
from auquanToolbox.dataloader import data_available

def loadgui(back_data, exchange, base_index, budget,logger):

	######################
	# Setup data
	######################

	position = back_data['POSITION']
	close = back_data['CLOSE']

	#position as % of total portfolio

	long_position = (position*close).div(back_data['VALUE'], axis = 0)
	short_position = long_position.copy()
	long_position[long_position < 0]= 0
	short_position[short_position > 0]= 0

	daily_pnl = back_data['DAILY_PNL']/budget
	total_pnl = back_data['TOTAL_PNL']/budget

	if base_index:
		baseline_data = baseline(exchange, base_index, total_pnl.index, logger)
		stats = metrics(daily_pnl, total_pnl, baseline_data,base_index)
	else:
		baseline_data={}
		stats = metrics(daily_pnl, total_pnl, {},base_index)


	daily_return = daily_pnl.sum(axis=1)
	total_return = total_pnl.sum(axis=1)
	long_exposure = long_position.sum(axis=1)
	short_exposure = short_position.sum(axis=1)
	zero_line = np.zeros(daily_pnl.index.size)

	#print to logger
	for x in stats.keys():
		logger.info('%s : %0.2f'%(x,stats[x]))


	def isDate(val):

		#Function to validate if a given entry is valid date
		try:
		    d = pd.to_datetime(val)
		    if d>daily_pnl.index[0] and d<daily_pnl.index[-1]:
		        return True
		    else:
		        return False
		except ValueError:
		    raise ValueError("Not a Valid Date")
		    return False

	def newselection(event):

		#Function to autoupdate chart on new selection from dropdown
		i = dropdown.current()
		market = ['TOTAL PORTFOLIO'] + daily_pnl.columns.values.tolist()
		plot(daily_pnl, total_pnl, long_position, short_position, baseline_data, base_index,market[i],box_value2.get(), box_value3.get())


	def plot(daily_pnl, total_pnl, long_position, short_position, baseline_data, base_index, market = 'TOTAL PORTFOLIO',start=daily_pnl.index.format()[0],end=daily_pnl.index.format()[-1]):

		#New plot when custom fields are changed

		plt.clf()

		#plt.style.use("seaborn-whitegrid")
		daily_pnl = daily_pnl.loc[start:end]
		total_pnl = total_pnl.loc[start:end]
		if base_index:
			base_pnl = baseline_data['TOTAL_PNL'].loc[start:end]
		long_position = long_position.loc[start:end]
		short_position = short_position.loc[start:end]

		if market == 'TOTAL PORTFOLIO':
		    daily_return = daily_pnl.sum(axis=1)
		    total_return = total_pnl.sum(axis=1)
		    long_exposure = long_position.sum(axis=1)
		    short_exposure = short_position.sum(axis=1)
		else:
		    daily_return = daily_pnl[market]
		    total_return = total_pnl[market]
		    long_exposure = long_position[market]
		    short_exposure = short_position[market]

		zero_line = np.zeros(daily_pnl.index.size)
		#f, plot_arr = plt.subplots(3, sharex=True)

		total_plot = plt.subplot2grid((10,8), (0,0), colspan = 12, rowspan = 4)
		daily_plot = plt.subplot2grid((10,8), (5,0), colspan = 12, rowspan = 2, sharex = total_plot)
		position_plot = plt.subplot2grid((10,8), (8,0), colspan = 12, rowspan = 2, sharex = total_plot)
		ind = np.arange(len(daily_pnl.index))
		      
		total_plot.set_title('Total PnL')
		total_plot.plot(ind, zero_line, 'k')
		total_plot.plot(ind, total_return.values, 'b',linewidth=0.5, label='strategy')
		total_plot.legend(loc='upper left')
		total_plot.autoscale(tight=True)
		plt.setp(total_plot.get_xticklabels(), visible=False)
		total_plot.yaxis.set_major_formatter(mtick.FuncFormatter(format_perc))    
		total_plot.set_ylabel('Cumulative Performance')
		total_plot.legend(bbox_to_anchor=(0.03, 0.97), loc='lower left', borderaxespad=0.)
		if base_index:
			total_plot.plot(ind, baseline_data['TOTAL_PNL'], 'g',linewidth=0.5, label=base_index)

		daily_plot.set_title('Daily PnL')
		daily_plot.plot(ind, zero_line, 'k')
		daily_plot.bar(ind, daily_return.values, 0.2, align='center', color='c', label='strategy')
		daily_plot.legend(loc='upper left')
		daily_plot.autoscale(tight=True)
		plt.setp(daily_plot.get_xticklabels(), visible=False)
		daily_plot.yaxis.set_major_formatter(mtick.FuncFormatter(format_perc))
		daily_plot.set_ylabel('Daily Performance')
		daily_plot.legend(bbox_to_anchor=(0.03, 0.97), loc='lower left', borderaxespad=0.)

		position_plot.set_title('Daily Exposure')
		position_plot.plot(ind, zero_line, 'k')
		position_plot.bar(ind, short_exposure.values, 0.3,linewidth=0, align='center', color='r', label='short')
		position_plot.bar(ind, long_exposure.values, 0.3,linewidth=0, align='center', color='b', label='long')
		position_plot.legend(loc='upper left')
		position_plot.autoscale(tight=True)    
		position_plot.xaxis.set_major_formatter(mtick.FuncFormatter(format_date))
		position_plot.yaxis.set_major_formatter(mtick.FuncFormatter(format_perc))    
		position_plot.set_ylabel('Long/Short %')    
		position_plot.legend(bbox_to_anchor=(0.03, 0.97), loc='lower left', borderaxespad=0.)

		plt.gcf().canvas.draw()

	def update_plot():

		#Callback Function for plot button
		try:
		    d1 = pd.to_datetime(box_value2.get())
		    d2 = pd.to_datetime(box_value3.get())
		    if d1>=daily_pnl.index[0] and d2<=daily_pnl.index[-1]:
		        plot(daily_pnl, total_pnl, long_position, short_position, baseline_data, base_index, box_value.get(), box_value2.get(), box_value3.get())
		    else:
		    	tkMessageBox.showinfo("Date out of Range", "Please enter a date from %s to %s"%(daily_pnl.index[0].strftime('%Y-%m-%d'),daily_pnl.index[-1].strftime('%Y-%m-%d')))
		except ValueError:
		    raise ValueError("Not a Valid Date")

	def close_window(): 

		#Callback function for Quit Button
		GUI.destroy()
		GUI.quit()

	def format_date(x, pos=None):

		#Format axis ticklabels to dates
		thisind = np.clip(int(x + 0.5), 0, len(daily_pnl.index) - 1)
		return daily_pnl.index[thisind].strftime('%b-%y')

	def format_perc(y, pos=None):

		#Format axis ticklabels to %
		if budget>1:
			return '{percent:.2%}'.format(percent=y)
		else:
			return y
		


	######################
	## GUI mainloop
	######################

	#Create widget
	GUI = tk.Tk()
	GUI.title('Backtest Results')

	#Create dropdown for market

	Label_1 = tk.Label(GUI, text="Trading Performance:")
	Label_1.grid(row = 0, column = 0, sticky = tk.EW)

	box_value = tk.StringVar()
	dropdown = ttk.Combobox(GUI, textvariable  = box_value, state = 'readonly')
	dropdown['values'] = ['TOTAL PORTFOLIO'] + daily_pnl.columns.values.tolist()
	dropdown.grid(row=0, column=1,sticky=tk.EW)
	dropdown.current(0)
	dropdown.bind('<<ComboboxSelected>>',newselection)

	#Create entry field for start date

	Label_2 = tk.Label(GUI, text="Start Date")
	Label_2.grid(row = 0, column = 2, sticky = tk.EW)

	box_value2 = tk.StringVar(GUI, value=daily_pnl.index.format()[0])
	start = tk.Entry(GUI, textvariable  = box_value2, validate='key', validatecommand=(GUI.register(isDate),'%P'))
	start.grid(row=0, column=3,sticky=tk.EW)

	#Create entry field for end date

	Label_3 = tk.Label(GUI, text="End Date")
	Label_3.grid(row = 0, column = 4, sticky = tk.EW)

	box_value3 = tk.StringVar(GUI, value=daily_pnl.index.format()[-1])
	end = tk.Entry(GUI, textvariable  = box_value3, validate='key', validatecommand=(GUI.register(isDate),'%P'))
	end.grid(row=0, column=5,sticky=tk.EW)

	#Create Plot button to reload chart

	button1 = tk.Button(GUI, text='PLOT', command=update_plot)
	button1.grid(row = 0, column = 6, sticky = tk.EW)

	#Create text widget with backtest results

	customFont1 = tkFont.Font(family="Helvetica", size=9, weight="bold")
	customFont2 = tkFont.Font(family="Helvetica", size=12)

	text = tk.Text(GUI,height=3, width=50, wrap=tk.WORD,bd=5, padx = 10, pady=5)
	text.grid(row=1,column=0, columnspan = 7, sticky=tk.EW)
	String1 = ''
	String2 = ''
	for y in stats.keys():
		String1 = String1 + y + '\t\t'
		x = stats[y]
		if budget>1 and 'Ratio' not in y:
			String2 = String2 +'{percent:.2%}'.format(percent=x)+ '\t\t'
		else:
			String2 = String2 +'%0.2f'%x + '\t\t'
	text.insert(tk.END, String1)
	text.insert(tk.END,'\n')
	text.insert(tk.END, String2)		
	text.tag_add("keys", "1.0", "1.end")
	text.tag_config("keys", font = customFont1)
	text.tag_add("values", "2.0", "2.end")
	text.tag_config("values", foreground="red", font = customFont2)

	#Create canvas to plot chart

	f = plt.figure(figsize = (16,8))
	canvas = FigureCanvasTkAgg(f, master=GUI)
	canvas.get_tk_widget().grid(row=2,column=0,columnspan = 7, rowspan = 1, sticky=tk.NSEW)
	toolbar_frame = tk.Frame(GUI) 
	toolbar_frame.grid(row=4,column=0,columnspan=7) 
	toolbar = NavigationToolbar2TkAgg( canvas, toolbar_frame )

	#plot 3 subplots for total position, daily position and exposure

	plt.style.use("seaborn-whitegrid")
	total_plot = plt.subplot2grid((10,8), (0,0), colspan = 12, rowspan = 4)
	daily_plot = plt.subplot2grid((10,8), (5,0), colspan = 12, rowspan = 2, sharex = total_plot)
	position_plot = plt.subplot2grid((10,8), (8,0), colspan = 12, rowspan = 2, sharex = total_plot)
	ind = np.arange(len(daily_pnl.index))
	      
	total_plot.set_title('Total PnL')
	total_plot.plot(ind, zero_line, 'k')
	total_plot.plot(ind, total_return.values, 'b',linewidth=0.5, label='strategy')
	total_plot.legend(loc='upper left')
	total_plot.autoscale(tight=True)
	plt.setp(total_plot.get_xticklabels(), visible=False)
	total_plot.yaxis.set_major_formatter(mtick.FuncFormatter(format_perc))    
	total_plot.set_ylabel('Cumulative Performance')
	total_plot.legend(bbox_to_anchor=(0.03, 0.97), loc='lower left', borderaxespad=0.)
	if base_index:
		total_plot.plot(ind, baseline_data['TOTAL_PNL'], 'g',linewidth=0.5, label=base_index)

	daily_plot.set_title('Daily PnL')
	daily_plot.plot(ind, zero_line, 'k')
	daily_plot.bar(ind, daily_return.values, 0.2, align='center', color='c', label='strategy')
	daily_plot.legend(loc='upper left')
	daily_plot.autoscale(tight=True)
	plt.setp(daily_plot.get_xticklabels(), visible=False)
	daily_plot.yaxis.set_major_formatter(mtick.FuncFormatter(format_perc))
	daily_plot.set_ylabel('Daily Performance')
	daily_plot.legend(bbox_to_anchor=(0.03, 0.97), loc='lower left', borderaxespad=0.)

	position_plot.set_title('Daily Exposure')
	position_plot.plot(ind, zero_line, 'k')
	position_plot.bar(ind, short_exposure.values, 0.3,linewidth=0, align='center', color='r', label='short')
	position_plot.bar(ind, long_exposure.values, 0.3,linewidth=0, align='center', color='b', label='long')
	position_plot.legend(loc='upper left')
	position_plot.autoscale(tight=True)    
	position_plot.xaxis.set_major_formatter(mtick.FuncFormatter(format_date))
	position_plot.yaxis.set_major_formatter(mtick.FuncFormatter(format_perc))    
	position_plot.set_ylabel('Long/Short')    
	position_plot.legend(bbox_to_anchor=(0.03, 0.97), loc='lower left', borderaxespad=0.)

	plt.gcf().canvas.draw()

	#Create Quit Button

	button2 = tk.Button(GUI, text='QUIT', command=close_window)
	button2.grid(row = 4, column = 6, sticky = tk.EW)

	GUI.mainloop()


from __future__ import absolute_import, division, print_function, unicode_literals
import numpy as np
import pandas as pd
from pandas.tseries.offsets import BDay
from auquanToolbox.dataloader import data_available
import matplotlib.pyplot as plt

def metrics(daily_pnl, total_pnl, baseline_data, base_index):

    stats = {}
    daily_return = daily_pnl.sum(axis=1)
    total_return = total_pnl.sum(axis=1)

    stats['Total Pnl'] = (total_pnl.iloc[total_pnl.index.size-1].sum())
    stats['Annual Return'] = annualized_return(daily_return)
    stats['Annual Vol']=annual_vol(daily_return)
    stats['Sharpe Ratio'] = sharpe_ratio(daily_return)
    stats['Sortino Ratio'] = sortino_ratio(daily_return)
    stats['Max Drawdown']=max_drawdown(daily_return)
    stats['Profit Factor']=profit_factor(daily_return)
    stats['Profitablity (%)']=profit_percent(daily_return)
    if base_index:
        stats['Base Return(%)'] = annualized_return(baseline_data['DAILY_PNL'])
        stats['Beta'] = beta(daily_return,baseline_data['DAILY_PNL'])

    for x in stats.keys():
        if np.isnan(stats[x]):
            del stats[x]

    return stats

def annualized_return(daily_return):
    total_return = daily_return.sum()
    total_days = daily_return.index.size
    return ((1 + total_return)**(252 / total_days) - 1)
    

def annualized_std(daily_return):
    return np.sqrt(252)*np.std(daily_return)

def annualized_downside_std(daily_return):
    mar = 0
    downside_return = daily_return.copy()
    downside_return[downside_return > 0]= 0
    return np.sqrt(252)*np.std(downside_return)

def annual_vol(daily_return):
    return annualized_std(daily_return)

def sharpe_ratio(daily_return):
    stdev = annualized_std(daily_return)
    if stdev == 0:
        return np.nan
    else:
        return annualized_return(daily_return)/stdev

def sortino_ratio(daily_return):
    stdev = annualized_downside_std(daily_return)
    if stdev == 0:
        return np.nan
    else:
        return annualized_return(daily_return)/stdev

def max_drawdown(daily_return):
    return np.max(np.maximum.accumulate(daily_return) - daily_return)

def beta(daily_return, baseline_daily_return):
    stdev = np.std(baseline_daily_return)
    if stdev == 0:
        return np.nan
    else:
        return np.corrcoef(daily_return, baseline_daily_return)[0,1]*np.std(daily_return)/stdev

def alpha(daily_return, baseline_daily_return,beta):
    return annualized_return(daily_return) - beta*annualized_return(baseline_daily_return)

def profit_factor(daily_return):
    downside_return = daily_return.copy()
    downside_return[downside_return > 0]= 0
    upside_return = daily_return.copy()
    upside_return[upside_return < 0]= 0
    if downside_return.sum() == 0:
        return 0
    return -(upside_return.sum())/(downside_return.sum())

def profit_percent(daily_return):
    total_return = daily_return.copy()
    total_return[total_return != 0]= 1
    upside_return = daily_return.copy()
    upside_return[upside_return < 0]= 0
    upside_return[upside_return > 0]= 1
    if total_return.sum() == 0:
        return 0
    return upside_return.sum()/total_return.sum()

def baseline(exchange, base_index, date_range,logger):
    features = ['OPEN', 'CLOSE']
    baseline_data = {}

    assert data_available(exchange, [base_index],logger)
    csv = pd.read_csv('%s/historicalData/%s.csv'%(exchange.lower(),base_index.lower()), index_col=0)
    csv.index = pd.to_datetime(csv.index)
    csv.columns = [col.upper() for col in csv.columns]
    csv = csv.reindex(index=csv.index[::-1])
    #features = [col.upper() for col in csv.columns]

    for feature in features:
        baseline_data[feature] = pd.Series(0, index=date_range)
        baseline_data[feature][base_index] = csv[feature][date_range]
    
    baseline_data['DAILY_PNL'] = pd.Series(0, index=date_range)
    baseline_data['TOTAL_PNL'] = pd.Series(0, index=date_range)

    open_start = baseline_data['OPEN'][base_index].iloc[1]
    for end in range(1, date_range.size):
        close_curr = baseline_data['CLOSE'][base_index].iloc[end]
        close_last = baseline_data['CLOSE'][base_index].iloc[end-1]
        if end == 1:
            close_last = open_start
        pnl_curr = (close_curr - close_last) / open_start

        baseline_data['DAILY_PNL'].iloc[end] = pnl_curr
        baseline_data['TOTAL_PNL'].iloc[end] = pnl_curr + baseline_data['TOTAL_PNL'].iloc[end - 1]

    return baseline_data            

def analyze(exchange, markets, back_data):
    plt.close('all')
    f, plot_arr = plt.subplots(2, sharex=True)
    plot_arr[0].set_title('Open')
    plot_arr[1].set_title('Close')
    for m in markets:
        plot_arr[0].plot(back_data['OPEN'].index, back_data['OPEN'][m], label=m)
        plot_arr[1].plot(back_data['OPEN'].index, back_data['CLOSE'][m], label=m)
    plot_arr[0].legend(loc='upper center')
    plot_arr[1].legend(loc='upper center')
    plt.show()


from __future__ import absolute_import, division, print_function, unicode_literals
import numpy as np
import pandas as pd
from pandas.tseries.offsets import BDay
import os
import csv
import json
import logging
import datetime as dt
from auquanToolbox.dataloader import load_data
from auquanToolbox.resultviewer import loadgui
from auquanToolbox.metrics import metrics, baseline
import matplotlib.pyplot as plt


def backtest(exchange, markets, trading_strategy, date_start, date_end, lookback, budget=1000000, verbose=False, base_index='SPX', trading_costs = True, json=False):

    logger = get_logger()

    if updateCheck():
        logger.warn('Your version of auquanToolbox is not the most updated.' +
            ' If you are using pip, please use \'pip install -U auquanToolbox\'.' + 
            ' If you downloaded the package, you need to go to https://github.com/Auquan/auquan-toolbox-python'+
            ' to redownload that package.')

    #Verify Settings

    try:
        assert(isinstance(lookback, int)),"Lookback is invalid"
    except AssertionError:
            logger.exception("Lookback is invalid")
            raise

    #Load data for backtest
   
    (back_data, date_range) = load_data(exchange, markets, date_start, date_end, lookback, budget, logger)
    logger.info('Initial funds: %0.2f'%budget)
    logger.info('------------------------------------')
    logger.info('Evaluating...')

    budget_curr = budget
    
    position_curr = None
    margin_curr = None
    cost_to_trade = None

    res = []
    start_index = -1

    for startDate in pd.date_range(start=date_start, end=date_end, freq='B'):
        if startDate not in date_range:
            logger.info(startDate.strftime('Trading date is a Holiday or data not present :%d %b %Y'))
            continue
        end = date_range.get_loc(startDate)
        if start_index < 0:
            start_index = end

        start = end - lookback
        if start < 0:
            start = 0

        if position_curr is None:
            position_curr = back_data['POSITION'].iloc[end-1]
            margin_curr = back_data['MARGIN'].iloc[end-1]
            cost_to_trade = position_curr*0

        # get order and verify
        lookback_data = {feature: data[start: end] for feature, data in back_data.items()}
        order = trading_strategy(lookback_data)
        try:
            assert((order['SIGNAL'].isin([-1,0,1])).all())
        except AssertionError:
            logger.info("Signal can only be -1(sell), 0(hold) or 1(buy)")
            raise
        try:
            assert((order['PRICE']>=0).all())
        except AssertionError:
            logger.info("Price cannot be negative")
            raise
        try:
            assert(order['WEIGHTS']>=0).all() 
        except AssertionError:
            logger.info("Please check weights. Weights cannot be negative and should sum to <= 1")
            raise

        if order['WEIGHTS'].sum()>1:
            order['WEIGHTS'] = order['WEIGHTS']/order['WEIGHTS'].sum()

        # evaluate new position based on order and budget
        
        try:
            price_curr = back_data['OPEN'].iloc[end].astype(float)
            open_curr = back_data['OPEN'].iloc[end].astype(float)
            close_curr = back_data['CLOSE'].iloc[end].astype(float)
            open_last = back_data['OPEN'].iloc[end-1].astype(float)
            close_last = back_data['CLOSE'].iloc[end-1].astype(float)
            high = back_data['HIGH'].iloc[end - 1].astype(float)
            low = back_data['LOW'].iloc[end - 1].astype(float)
        except ValueError:
            logger.info("Data not formatted properly")
            raise


        slippage = (high - low) * 0.05
        position_last = back_data['POSITION'].iloc[end - 1].astype(int)
        value = budget_curr + margin_curr + (position_last * open_curr).sum()
        order['QUANTITY'] = getquantity(order, price_curr, slippage,value,position_last, logger)
        (position_curr, budget_curr, margin_curr, cost_to_trade) = execute_order(order, position_last, slippage, price_curr, budget_curr,margin_curr,logger, trading_costs)

        # set info in back data
        back_data['POSITION'].iloc[end] = position_curr
        back_data['ORDER'].iloc[end] = order['QUANTITY']
        filled_order = position_curr - position_last
        back_data['FILLED_ORDER'].iloc[end] = filled_order

        # calculate pnl
        pnl_curr = (position_curr * (close_curr  - open_curr) + position_last * (open_curr - close_last)) - cost_to_trade
        back_data['DAILY_PNL'].iloc[end] = pnl_curr
        back_data['TOTAL_PNL'].iloc[end] = pnl_curr + back_data['TOTAL_PNL'].iloc[end - 1]

        # available funds
        back_data['FUNDS'].iloc[end] = budget_curr

        #funds used as margin
        back_data['MARGIN'].iloc[end] = -(position_curr[position_curr<0] * close_curr[position_curr<0]).sum()

        #portfolio value
        value_curr = budget_curr + margin_curr+ (margin_curr - back_data['MARGIN'].iloc[end]) + (position_curr[position_curr>0] * close_curr[position_curr>0]).sum()
        back_data['VALUE'].iloc[end] = value_curr

        #cost
        back_data['COST TO TRADE'].iloc[end] = cost_to_trade

        #print to STDOUT
        logger.info(date_range[end].strftime('Trading date :%d %b %Y'))
        if verbose:
            s = 'stocks         : %s'%markets+'\n'+\
            'today open     : %s'%open_curr.values+'\n'+\
            'today close    : %s'%close_curr.values+'\n'+\
            'order          : %s'%order['QUANTITY'].values+'\n'+\
            'position       : %s'%position_curr.values+'\n'+\
            'cost to trade  : %0.2f'%cost_to_trade.sum()+'\n'+\
            'Available funds: %0.2f'%budget_curr+'\n'+\
            'Margin funds   : %0.2f'%margin_curr+'\n'+\
            'pnl            : %0.2f'%pnl_curr.sum()+'\n'+\
            'Portfolio Value: %0.2f'%value_curr+'\n'+\
            '------------------------------------'
            logger.info(s)
        
        if value_curr<=0:
            logger.info('Out of funds. Exiting!')
            break
            
    logger.info('Final Portfolio Value: %0.2f'%value_curr)
    #writejson(back_data,budget)

    if json:
        if base_index:
            baseline_data = baseline(exchange, base_index, date_range, logger)
            return writejson({feature: data[start_index-1: end+1] for feature, data in back_data.items()},budget,{feature: data[start_index-1: end+1] for feature, data in baseline_data.items()}, base_index)
        else:
            return writejson({feature: data[start_index-1: end+1] for feature, data in back_data.items()},budget,{}, base_index)
    else:
        writecsv({feature: data[start_index-1: end+1] for feature, data in back_data.items()},budget)

    logger.info('Plotting Results...')

    loadgui({feature: data[start_index-1: end+1] for feature, data in back_data.items()}, exchange, base_index, budget,logger)

def commission():
    return 0.1

def margin_perc():
    return 1

def getquantity(order, price, slippage,value,position,logger):
    weights = order['WEIGHTS']
    cost_to_trade = slippage+commission()
    if weights.sum()>0:
        new_portfolio_value = (weights.sum()*value)/(weights*(price+cost_to_trade)/price).sum()
        desired_position = weights*new_portfolio_value/price
        quantity = (order['SIGNAL']*desired_position) - position
    else:
        new_portfolio_value = 0
        quantity = - position
        quantity.fillna(0)
    return quantity.astype(int)

def execute_order(order, position, slippage, price, budget, margin, logger, trading_costs):

    trade_criteria = (np.sign(order['QUANTITY'])*price[order.index] <= np.sign(order['QUANTITY'])*order['PRICE'])
    trade_criteria[np.sign(order['QUANTITY'])*price[order.index] > np.sign(order['QUANTITY'])*order['PRICE']] = order['PRICE'] ==0
    
    position_curr = position.copy()
    total_commission = 0*position_curr
    adj_slippage = 0*position_curr

    position_curr[trade_criteria] += order['QUANTITY'][trade_criteria]
    margin_curr = -(position_curr[position_curr < 0] * price[position_curr < 0]).sum()
    if trading_costs:
        total_commission = np.abs(position_curr - position) * commission()
        slippage_adjusted_price = price + (np.sign(order['QUANTITY'])*slippage)
        slippage_adjusted_price[slippage_adjusted_price < 0] = 0
        adj_slippage = np.abs(position_curr - position)*np.abs(price - slippage_adjusted_price)
    margin_call = margin_curr-margin
    order_value = ((position_curr - position) * price).sum() + margin_call
    cost_to_trade = total_commission + adj_slippage
    return position_curr, budget - order_value - margin_call - cost_to_trade.sum(), margin_curr, cost_to_trade

def get_logger():
    logger_name = dt.datetime.now().strftime('%Y-%m-%d %H-%M-%S')
    logger = logging.getLogger(logger_name)
    logger.setLevel(logging.DEBUG)
    logger_dir = 'runLogs/'
    logger_file = '%srun-%s.txt'%(logger_dir,logger_name)
    if not os.path.exists(logger_dir):
        os.makedirs(logger_dir)
    formatter = logging.Formatter('%(message)s')
    file_handler = logging.FileHandler(logger_file)
    console_handler = logging.StreamHandler()
    file_handler.setFormatter(formatter)
    console_handler.setFormatter(formatter)
    logger.addHandler(file_handler)
    logger.addHandler(console_handler)
    return logger

def writecsv(back_data,budget):

    results = pd.DataFrame(0, index=back_data['DAILY_PNL'].index, columns=['Daily Returns'])
    results['Daily Returns'] = back_data['DAILY_PNL'].sum(axis=1)*100/budget
    results['Total Returns'] = back_data['TOTAL_PNL'].sum(axis=1)*100/budget
    results['Funds'] = back_data['FUNDS']
    results['Margin'] = back_data['MARGIN']
    results['Portfolio Value'] = back_data['VALUE']
    for stock in back_data['DAILY_PNL'].columns.tolist():
        results['%s Position'%stock]= back_data['POSITION'][stock]
        results['%s Order'%stock]=back_data['ORDER'][stock]
        results['%s Filled Order'%stock]=back_data['FILLED_ORDER'][stock]
        results['%s Trade Price'%stock]=back_data['OPEN'][stock]
        results['%s Cost to Trade'%stock]=back_data['COST TO TRADE'][stock]
        results['%s PnL'%stock]=back_data['DAILY_PNL'][stock]

    results = results.sort_index(axis=0,ascending=False)
    csv_dir = 'runLogs/'
    try:
        csv_file =  open('%srun-%s.csv'%(csv_dir, dt.datetime.now().strftime('%Y-%m-%d %H-%M-%S')), 'wb')
        results.to_csv(csv_file)
    except:
        csv_file =  open('%srun-%s.csv'%(csv_dir, dt.datetime.now().strftime('%Y-%m-%d %H-%M-%S')), 'w')
        results.to_csv(csv_file)
    # writer = csv.writer(csv_file)
    # writer.writerow(['Dates']+back_data['DAILY_PNL'].index.format())
    # writer.writerow(['Daily Pnl']+daily_return.sum(axis=1).values.tolist())
    # writer.writerow(['Total PnL']+total_return.sum(axis=1).values.tolist())
    # writer.writerow(['Funds']+back_data['FUNDS'].values.tolist())
    # writer.writerow(['Portfolio Value']+back_data['VALUE'].values.tolist())
    # for stock in back_data['DAILY_PNL'].columns.tolist():
    #     writer.writerow(['%s Position'%stock]+back_data['POSITION'][stock].values.tolist())
    #     writer.writerow(['%s Order'%stock]+back_data['ORDER'][stock].values.tolist())
    #     writer.writerow(['%s Filled Order'%stock]+back_data['FILLED_ORDER'][stock].values.tolist())
    #     writer.writerow(['%s Slippage'%stock]+back_data['SLIPPAGE'][stock].values.tolist())
    #     writer.writerow(['%s PnL'%stock]+back_data['DAILY_PNL'][stock].values.tolist())
    csv_file.close()

def writejson(back_data,budget,baseline_data,base_index):

    daily_return = back_data['DAILY_PNL']*100/budget
    total_return = back_data['TOTAL_PNL']*100/budget
    stats = metrics(daily_return, total_return, baseline_data,base_index)

    d = {'dates':back_data['DAILY_PNL'].index.format(),\
         'daily_pnl':daily_return.sum(axis=1).values.tolist(),\
         'total_pnl':total_return.sum(axis=1).values.tolist(),\
         'stocks':back_data['DAILY_PNL'].columns.tolist(),\
         'stock_pnl':daily_return.values.tolist(),\
         'stock_position':back_data['POSITION'].values.tolist(),\
         'metrics':stats.keys(),\
         'metrics_values':stats.values()}
    return d;


def updateCheck():
    ''' checks for new version of toolbox
    Returns:
        returns True if the version of the toolox on PYPI is not the same as the current version
        returns False if version is the same
    '''

    from auquanToolbox.version import __version__
    updateStr = ''
    try:
        toolboxJson = urllib.urlopen('https://pypi.python.org/pypi/auquanToolbox/json')
    except Exception as e:
        return False

    toolboxDict = json.loads(toolboxJson.read())

    if __version__ != toolboxDict['info']['version']:
        return True
    else:
        return False



from __future__ import absolute_import, division, print_function, unicode_literals
import pandas as pd
import numpy as np
import auquanToolbox as at

def settings():
    exchange = "nasdaq"           # Exchange to download data for (only nasdaq for now)
    markets = ['A','AAPL','IBM','GOOG','C'] # Stocks to download data for. 
    # markets = [] Leave empty array to download all stocks for the exchange (~900 stocks)
    # To have a look at all possible stocks go here: 
    # https://raw.githubusercontent.com/Auquan/auquan-historical-data/master/nasdaq/nasdaq.txt
    # Based on data avalaible, some markets might be dropped. Only the non dropped markets will appear
    # in lookback_data for trading_strategy

    date_start = '2015-01-03'   # Date to start the backtest
    date_end = '2016-11-06'     # Date to end the backtest
    lookback = 120               # Number of days you want historical data for

    """ To make a decision for day t, your algorithm will have historical data
    from t-lookback to t-1 days"""
    return [exchange, markets, date_start, date_end, lookback]

def trading_strategy(lookback_data):
    """
    :param lookback_data: Historical Data for the past "lookback" number of days as set in the main settings.
     It is a dictionary of features such as,
     'OPEN', 'CLOSE', 'HIGH', 'LOW', 'VOLUME', 'SLIPPAGE', 'POSITION', 'ORDER',
     'FILLED_ORDER', 'DAILY_PNL', 'TOTAL_PNL', 'FUNDS', 'VALUE'
     Any feature data can be accessed as:lookback_data['OPEN']
     The output is a pandas dataframe with dates as the index (row)
     and markets as columns. 
    """"""""""""""""""""""""""
    """""""""""""""""""""""""" 
    To see a complete list of features, uncomment the line below"""
    #print(lookback_data.keys())
    
    """""""""""""""""""""""""""
    :return: A pandas dataframe with markets you are trading as index(row) and 
    signal, price and quantity as columns
    order['SIGNAL']:buy (+1), hold (0) or sell (-1) trading signals for all securities in markets[]
    order['PRICE']: The price where you want to trade each security. Buy orders are executed at or below the price and sell orders are executed at or above the price
    order['WEIGHTS']: The normalized set of weights for the markets
    System will buy the specified quantity of stock if it's price <= price specified here
    System will sell the specified quantity of a stock if it's price >= price specified here
    """

    """IMPORTANT: Please make sure you have enough funds to buy or sell. 
        Order is cancelled if order_value > available funds(both buy and short sell)"""

    order = pd.DataFrame(0, index=lookback_data['POSITION'].columns, columns = ['SIGNAL','WEIGHTS','PRICE'])

    ##YOUR CODE HERE

    return order

if __name__ == '__main__':
    [exchange, markets, date_start, date_end, lookback] = settings()
    at.backtest(exchange, markets, trading_strategy, date_start, date_end, lookback)#,verbose=True)



from __future__ import absolute_import, division, print_function, unicode_literals
try:
    from urllib2 import urlopen
except ImportError:
    from urllib.request import urlretrieve, urlopen
import numpy as np
import pandas as pd
from pandas.tseries.offsets import BDay
import os

def download(exchange, ticker, file_name,logger):
    url = 'https://raw.githubusercontent.com/Auquan/auquan-historical-data/master/%s/historicalData/%s.csv'%(exchange.lower(), ticker.lower())
    response = urlopen(url)
    status = response.getcode()
    if status == 200:
        logger.info('Downloading %s data to file: %s'%(ticker, file_name))
        with open(file_name, 'w') as f: f.write(response.read())
        return True
    else:
        logger.info('File not found. Please check settings!')
        return False

def data_available(exchange, markets,logger):
    dir_name = '%s/historicalData/'%exchange.lower()
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)
    for m in markets:
        file_name = '%s%s.csv'%(dir_name, m.lower())
        if not os.path.exists(file_name):
            try:
                assert(download(exchange, m, file_name,logger)),"%s not found. Please check settings!"%file_name
            except AssertionError:
                logger.exception("%s not found. Please check settings!"%file_name)
                raise      
    return True

def download_security_list(exchange, logger):
    dir_name = '%s/'%exchange.lower()
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)

    file_name = '%s%s.txt'%(dir_name, exchange.lower())
    if not os.path.exists(file_name):
        url = 'https://raw.githubusercontent.com/Auquan/auquan-historical-data/master/%s'%(file_name)
        status = urlopen(url).getcode()
        if status == 200:
            logger.info('Downloading data to file: %s'%file_name)
            urlretrieve(url, file_name)
            return True
        else:
            logger.info('File not found. Please check exchange settings!')
        return False
    else:
        return True

def compatibleDictKeyCheck(dict, key):
    try:
        return dict.has_key(key)
    except:
        return key in dict

def load_data(exchange, markets, start, end, lookback, budget, logger, random=False):

    logger.info("Loading Data from %s to %s...."%(start,end))

    # because there are some holidays adding some cushion to lookback
    try:
        dates = [pd.to_datetime(start)-BDay(lookback* 1.10), pd.to_datetime(end)]
    except ValueError:
        logger.exception("%s or %s is not valid date. Please check settings!"%(start, end))
        raise ValueError("%s or %s is not valid date. Please check settings!"%(start, end))

    try:
        assert(dates[1]>dates[0]),"Start Date is after End Date"
    except AssertionError:
        logger.exception("Start Date is after End Date")
        raise

    #Download list of securities
    assert(download_security_list(exchange, logger))
    if len(markets)==0:
        file_name = '%s/%s.txt'%(exchange.lower(), exchange.lower())
        
        markets = [line.strip() for line in open(file_name)]
 

    markets = [m.upper() for m in markets]
    features = ['OPEN', 'CLOSE', 'HIGH', 'LOW', 'VOLUME']
    date_range = pd.date_range(start=dates[0], end=dates[1], freq='B')
    back_data = {}
    if random:
        for feature in features:
            back_data[feature] = pd.DataFrame(np.random.randint(10, 50, size=(date_range.size,len(markets))),
                                              index=date_range,
                                              columns=markets)
    else:
        for feature in features:
            back_data[feature] = pd.DataFrame(index=date_range, columns=markets)
        assert data_available(exchange, markets, logger)
        market_to_drop = []
        for market in markets:
            logger.info('Reading %s.csv'%market)
            csv = pd.read_csv('%s/historicalData/%s.csv'%(exchange.lower(), market.lower()), index_col=0)
            csv.index = pd.to_datetime(csv.index)
            csv.columns = [col.upper() for col in csv.columns]
            csv = csv.reindex(index=csv.index[::-1])
            features = [col.upper() for col in csv.columns]
            market_first_date = csv.index[0]
            if (market_first_date > (dates[0]-BDay(1)+BDay(1))):
                market_to_drop.append(market)
                logger.info('Dropping %s. This stock did not start trading before (start date -lookback days)'%market)
                continue
            market_last_date = csv.index[-1]
            if (market_last_date < (dates[0] - BDay(1) + BDay(1))):
                market_to_drop.append(market)
                logger.info('Dropping %s. This stock terminated before (start date -lookback days)'%market)
                continue

            back_fill_data = False
            if market_last_date in date_range:
                back_fill_data = True
                logger.info('The market %s doesnt have data for the whole duration. Subsituting missing dates with the last known data'%market)

            for feature in features:
                back_data[feature][market] = csv[feature][date_range]
                if back_fill_data:
                    back_data[feature].loc[market_last_date:date_range[-1], market] = back_data[feature].at[market_last_date, market]

        for m in market_to_drop: 
            logger.info('Dropping %s. Not Enough Data'%m)
            markets.remove(m) 

        for feature in features:
            back_data[feature].drop(market_to_drop, axis=1, inplace=True)
        dates_to_drop = pd.Series(False, index=date_range)
        for feature in features:
            dates_to_drop |= pd.isnull(back_data[feature]).any(axis=1)

        dropped_dates = date_range[dates_to_drop]
        date_range = date_range[~dates_to_drop]
        for feature in features:
            back_data[feature] = back_data[feature].drop(dropped_dates)

    back_data['COST TO TRADE'] = pd.DataFrame(0, index=date_range, columns=markets)
    back_data['POSITION'] = pd.DataFrame(0, index=date_range, columns=markets)
    back_data['ORDER'] = pd.DataFrame(0, index=date_range, columns=markets)
    back_data['FILLED_ORDER'] = pd.DataFrame(0, index=date_range, columns=markets)
    back_data['DAILY_PNL'] = pd.DataFrame(0, index=date_range, columns=markets)
    back_data['TOTAL_PNL'] = pd.DataFrame(0, index=date_range, columns=markets)
    back_data['FUNDS'] = pd.Series(budget, index=date_range)
    back_data['VALUE'] = pd.Series(budget, index=date_range)
    back_data['MARGIN'] = pd.Series(0, index=date_range)

    return back_data, date_range

# TODO: Refactor this
def load_data_nologs(exchange, markets, start, end, lookback=2):

    # because there are some holidays adding some cushion to lookback
    try:
        dates = [pd.to_datetime(start)-BDay(lookback* 1.10), pd.to_datetime(end)]
    except ValueError:
        raise ValueError("%s or %s is not valid date. Please check settings!"%(start, end))

    assert(dates[1]>dates[0]),"Start Date is after End Date"

    dir_name = '%s/'%exchange.lower()
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)

    file_name = '%s%s.txt'%(dir_name, exchange.lower())
    if not os.path.exists(file_name):
        url = 'https://raw.githubusercontent.com/Auquan/auquan-historical-data/master/%s'%(file_name)
        status = urlopen(url).getcode()
        if status == 200:
            urlretrieve(url, file_name)
        else:
            print('File not found. Please check exchange name!')

    if len(markets)==0:
        file_name = '%s/%s.txt'%(exchange.lower(), exchange.lower())
        
        markets = [line.strip() for line in open(file_name)]
 

    markets = [m.upper() for m in markets]
    features = ['OPEN', 'CLOSE', 'HIGH', 'LOW', 'VOLUME']
    date_range = pd.date_range(start=dates[0], end=dates[1], freq='B')
    back_data = {}
    for feature in features:
        back_data[feature] = pd.DataFrame(index=date_range, columns=markets)
    dir_name = '%s/historicalData/'%exchange.lower()
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)
    for m in markets:
        file_name = '%s%s.csv'%(dir_name, m.lower())
        if not os.path.exists(file_name):
            url = 'https://raw.githubusercontent.com/Auquan/auquan-historical-data/master/%s/historicalData/%s.csv'%(exchange.lower(), m.lower())
            status = urlopen(url).getcode()
            if status == 200:
                urlretrieve(url, file_name)
            else:
                print('File not found. Please check settings!')
    
    market_to_drop = []
    for market in markets:
        csv = pd.read_csv('%s/historicalData/%s.csv'%(exchange.lower(), market.lower()), index_col=0)
        csv.index = pd.to_datetime(csv.index)
        csv.columns = [col.upper() for col in csv.columns]
        csv = csv.reindex(index=csv.index[::-1])
        features = [col.upper() for col in csv.columns]
        market_first_date = csv.index[0]
        if (market_first_date > (dates[0]-BDay(1)+BDay(1))):
            market_to_drop.append(market)
            continue
        market_last_date = csv.index[-1]
        if (market_last_date < (dates[0] - BDay(1) + BDay(1))):
            market_to_drop.append(market)
            continue

        back_fill_data = False
        if market_last_date in date_range:
            back_fill_data = True

        for feature in features:
            back_data[feature][market] = csv[feature][date_range]
            if back_fill_data:
                back_data[feature].loc[market_last_date:date_range[-1], market] = back_data[feature].at[market_last_date, market]

        for m in market_to_drop: 
            markets.remove(m) 

        for feature in features:
            back_data[feature].drop(market_to_drop, axis=1, inplace=True)
        dates_to_drop = pd.Series(False, index=date_range)
        for feature in features:
            dates_to_drop |= pd.isnull(back_data[feature]).any(axis=1)

        dropped_dates = date_range[dates_to_drop]
        date_range = date_range[~dates_to_drop]
        for feature in features:
            back_data[feature] = back_data[feature].drop(dropped_dates)

    return back_data



from setuptools import setup

from auquanToolbox.version import __version__


setup(name='auquanToolbox',
      version=__version__,
      description='The Auquan Toolbox for trading system development',
      url='http://auquan.com/',
      author='Auquan',
      author_email='info@auquan.com',
      license='MIT',
      packages=['auquanToolbox'],
      scripts=['TradingStrategyTemplate.py'],
      include_package_data = True,

      install_requires=[
        'pandas',
        'numpy',
        'matplotlib',
      ],

      zip_safe=False,
     )